<?php
/*******************************************************************************
 *	BDirectoryLister.php	Directory Lister class
 *				T.Barnaby,	BEAM Ltd,	19/1/05
 *******************************************************************************
 */

require_once "BHtml.php";

class BFileListEntry {
	function BFileListEntry($filePattern, $title){
		$this->filePattern = $filePattern;
		$this->title = $title;
	}
	var	$filePattern;
	var	$title;
};

function patternCompareArray($array, $value){
	foreach($array as $p){
		if(preg_match("/^" . $p . "$/", $value)){
			return 1;
		}
	}
	return 0;
}

class BDirectoryLister {
	function BDirectoryLister(){
	}

	function getIcon($fileName){
		global $state;
		
		$baseUrl = $state->baseUrl;
		
		$ext = pathinfo($fileName, PATHINFO_EXTENSION);
		if(is_dir($fileName)){
			return $baseUrl . "/icons/filetypes/folder.gif";
		}
		else if($ext == "pdf"){
			return $baseUrl . "/icons/filetypes/pdf.gif";
		}
		else if($ext == "rpm"){
			return $baseUrl . "/icons/filetypes/compressed.gif";
		}
		else if($ext == "gz"){
			return $baseUrl . "/icons/filetypes/compressed.gif";
		}
		else if($ext == "txt"){
			return $baseUrl . "/icons/filetypes/text.gif";
		}
		else if($ext == "cd"){
			return $baseUrl . "/icons/filetypes/image1.gif";
		}
		else {
			return $baseUrl . "/icons/filetypes/unknown.gif";
		}
	}

	function getDirListByName($path, $excludes){
		// Open the folder
		$dir = @opendir($path) or die("Unable to open $path");

		// Get list of files the files
		$dlist = array();
		while($file = readdir($dir)){
			if(!patternCompareArray($excludes, $file)){
				$dlist[$file] = $file;
			}
		}

		// Close
		closedir($dir);
		
		// Sort into date order
		asort($dlist);

		return $dlist;
	}

	function getDirListByDate($path, $excludes){
		// Open the folder
		$dir = @opendir($path) or die("Unable to open $path");

		// Get list of files the files
		$dlist = array();
		while($file = readdir($dir)){
			if(!patternCompareArray($excludes, $file)){
				$dlist[$file] = filemtime($path . $file);
			}
		}

		// Close
		closedir($dir);
		
		// Sort into date order
		arsort($dlist);

		return $dlist;
	}

	function createList($path){
		$excludes = array("\.", "\.\.", "index.php", "FileList.bp");
		
		$table = new BHtmlNode("table", "cellpadding='4' cellspacing='4' border='0'");
		
		// Add headers
		$r = &$table->appendNode("tr");
		$c = &$r->appendNode("th");
		$c->appendText("Type");
		$c = &$r->appendNode("th");
		$c->appendText("FileName");
		$c = &$r->appendNode("th");
		$c->appendText("Date");
		$c = &$r->appendNode("th");
		$c->appendText("Size");

		$r = &$table->appendNode("tr");
		$c = &$r->appendNode("td", "colspan=4");
		$c->appendNode("hr");
		
		// Get the directory list
		$dlist = $this->getDirListByName($path, $excludes);
		
		// Loop through the files
		foreach($dlist as $file){
			$icon = $this->getIcon($path . "/" . $file);

			$r = &$table->appendNode("tr");
			$c = &$r->appendNode("td");
			$l = &$c->appendNode("img", "src=\"$icon\"");

			$c = &$r->appendNode("td");
			$l = &$c->appendNode("a", "href=\"$file\"");
			$l->appendText($file);

			$c = &$r->appendNode("td");
			$c->appendText(date("Y-m-d h:ia", filemtime($path . $file)));

			$c = &$r->appendNode("td", "align=right");
			$c->appendText(strval(intval(filesize($path . $file) / 1024)) . "K");
		}

		return $table->render();
	}

	function createListFromFile($path, $fileList){
		$title = "";
		$excludes = array("\.", "\.\.", "index.php", "FileList.bp");
		
		// Get FileList entries for extra file info
		$f = fopen($fileList, "r");
		while($l = fgets($f)){
			$ls = explode(":", $l);
			if($ls[0] == "Exclude"){
				$excludes[] = $ls[1];
			}
			else if($ls[0] == "Header"){
				$header = file_get_contents($path . "/" . $ls[1]);
			}
			else if($ls[0] == "File"){
				$fl[] = $a = new BFileListEntry($ls[1], $ls[2]);
			}
		}

		$table = new HtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
		
		// Add headers
		$r = &$table->appendNode("tr");
		$c = &$r->appendNode("th");
		$c->appendText("Type");
		$c = &$r->appendNode("th");
		$c->appendText("FileName");
		$c = &$r->appendNode("th");
		$c->appendText("Date");
		$c = &$r->appendNode("th");
		$c->appendText("Size");
		$c = &$r->appendNode("th");
		$c->appendText("Title");

		// Get the directory list
		$dlist = $this->getDirListByDate($path, $excludes);
		
		foreach($dlist as $file => $fileTime){
			$icon = $this->getIcon($path . "/" . $file);

			// Try and match name
			$title = "";
			foreach($fl as $fe){
				if(preg_match("/^" . $fe->filePattern . "$/", $file)){
					$title = $fe->title;
				}
			}

			$r = &$table->appendNode("tr");
			$c = &$r->appendNode("td");
			$l = &$c->appendNode("img", "src=\"$icon\"");

			$c = &$r->appendNode("td");
			$l = &$c->appendNode("a", "href=\"$file\"");
			$l->appendText($file);

			$c = &$r->appendNode("td");
			$c->appendText(date("Y-m-d h:ia", filemtime($path . $file)));

			$c = &$r->appendNode("td", "align=right");
			$c->appendText(strval(intval(filesize($path . $file) / 1024)) . "K");

			$c = &$r->appendNode("td");
			$c->appendText($title);
			$c->appendNode("br");
		}

		return $header . $table->render();
	}

};
?>
