<?php
/*******************************************************************************
 *	BPage.php		Main page output
 *				Generates a HTML page for display
 *				T.Barnaby,	BEAM Ltd,	2008-02-08
 *******************************************************************************
 */
require_once "BMenu.php";
require_once "BMenuRender.php";


class BPage {
	function BPage($state){
		$this->state = $state;
		$this->leftObjects = array();
		$this->setLogo($state->logo);
		$this->setTitle($state->title);
		$this->print = 0;
		$this->popup = 0;
	}
	
	// Create standard layout
	function init(){
	}

	function setLogo($logo){
		$this->logo = $logo;

		if($logo[0] == '/')
			$this->logoHtml = "{$this->state->baseUrl}$logo";
		else if(file_exists("{$this->state->baseDir}/themes/{$this->state->style}/$logo"))
			$this->logoHtml = "{$this->state->baseUrl}/themes/{$this->state->style}/$logo";
		else if(file_exists("{$this->state->baseDir}/icons/$logo"))
			$this->logoHtml = "{$this->state->baseUrl}/icons/$logo";
		else
			$this->logoHtml = $logo;
	}
	
	function setTitle($title){
		$this->title = $title;
	}

	function setMenuBar($menu){
		$this->menuMain = $menu;
		$this->menuMain->rebaseMenu();
		$this->menuBarHtml = menuBarRender($this->menuMain);
	}

	function setStatus($status){
		$this->status = $status;
	}

	function leftObjectInsert($obj){
		if($obj instanceof BMenu)
			$obj->rebaseMenu();
		array_splice($this->leftObjects, 0, 0, array($obj));
	}

	function setCentre($center){
		$this->center = $center;
	}
	
	// Render the page
	function render(){
	}

	
	// Override standard layout bits
	function setTop($top){
		$this->top = $top;
	}
	
	function setTopMenu($topMenu){
		$this->topMenu = $topMenu;
	}
	
	function setLeftSide($leftSide){
		$this->leftSide = $leftSide;
	}
	
	function setBottom($bottom){
		$this->bottom = $bottom;
	}
	

	// Page settings
	var	$state;								// The overal state
	var	$logo;								// The Logo to use
	var	$title;								// The page title
	var	$mainMenu;							// The main menu
	var	$location;
	var	$leftObjects;							// The list of objects on the LHS

	// Special functions
	var	$print;				// Sets page for printing
	var	$popup;				// Sets page for popup

	// The page elements
	var	$top;
	var	$topMenu;
	var	$status;
	var	$leftSide;
	var	$center;
	var	$bottom;
	
	// Internal bits
	var	$logoHtml;
	var	$menuBarHtml;							// The Rendered menubar
};

require "Smarty.class.php";

class BPageTemplate extends BPage {
	function BPageTemplate($state){
		BPage::BPage($state);
		
		$this->smarty = new Smarty;
		$this->smarty->compile_check = true;				// Recompile if templates have changed.
		$this->smarty->force_compile = true;
		$this->smarty->use_sub_dirs  = true;
		$this->smarty->debugging     = false;				// Popup Javascript window with template vars.				
		$this->smarty->caching       = 0;
		$this->smarty->template_dir	= $this->state->baseDir . "/themes";
		$this->smarty->compile_dir	= "/tmp/templates_c/" . $this->state->siteName;

		if(!file_exists($this->smarty->compile_dir))
			mkdir($this->smarty->compile_dir, 0777, 1);
	}

	function templateName($baseName){
		if($this->state->style == Null){
			return $this->state->baseDir . "/themes/default/" . $baseName;
		}
		else {
			return $this->state->baseDir . "/themes/" . $this->state->style . "/" . $baseName;
		}
	}
	
	function render(){
		if($this->print){
			print $this->center;
		}
		else if($this->popup){
			$this->smarty->assign("this", $this);
			$this->smarty->assign("BPageParam", $this);
			$this->smarty->display($this->templateName("popup.tpl"));
		}
		else {
			$this->smarty->assign("this", $this);
			$this->top		= $this->renderTemplate($this->templateName("top.tpl"));
			$this->topMenu		= $this->renderTemplate($this->templateName("topMenu.tpl"));
			$this->status		= $this->renderTemplate($this->templateName("status.tpl"));
			$this->leftSide		= $this->renderTemplate($this->templateName("leftSide.tpl"));
			$this->bottom		= $this->renderTemplate($this->templateName("bottom.tpl"));

			$this->smarty->assign("BPageParam", $this);
			$this->smarty->display($this->templateName("page.tpl"));
		}
	}
	
	// Aditional variables
	function renderTemplate($template, $param = ""){
		if($param != "")
			$this->smarty->assign("param", $param);
		return $this->smarty->fetch($template);
	}
	
	// Template specific variables
	var	$smarty;
};

?>
